#!/usr/bin/python3
#This script starts the Runtime "GrafcetEngine". Grafcet-Studio can then connect to the device. See www.mhj-wiki.de

from GrafcetEngineRuntime import GrafcetEngine #import Grafcet-Engine
import time

#Offset of hardware I/O and length
di_ofs=11 #ofs of DIs of the Revolution Pi (see export file of PICtory)
di_len=2 #2 Bytes DI
do_ofs=81 #ofs of DOs of the Revolution Pi (see export file of PICtory)
do_len=2 #2 Bytes DO

grafcet_di_ofs=0 #ofs address of the DIs in the grafcet engine (see symbolic table)
grafcet_do_ofs=25 #ofs address of the DOs in the grafcet engine (see symbolic table)

engine=GrafcetEngine() #Create an instance of the Grafcet Engine

#Init the communication over MQTT
#!!! IMPORTANT !!! This settings MUST match with the settings in Grafcet-Studio: IP address, Topic1 and Topic 2
#parameters: url of broker, topic1 and topic2 , 
mqttOk=engine.InitMqtt("broker.mqttdashboard.com","mh","revolutionpi") #change Topic1 and Topic 2: at least 3 characters
if mqttOk: print("MQTT OK") #is ok
else:print("!!!!! MQTT Error") #error

timeOldStatusMessage=time.time() #for status messages
f = open("/dev/piControl0","wb+",0) #access the process image of the revolution pi

#Loop for processing 
print("Start Grafcet loop...")
try:
    while True: #loop
            #read hardware inputs and write these values into the inputs of the Grafcet-Engine
            #DI
            f.seek(di_ofs)   # offset of the DI is 11
            di_array=f.read(di_len) #read 2 bytes from the process image of the RevPi
            for i in range(di_len):
                engine.write_uint8(grafcet_di_ofs+i, di_array[i])
            #Call Grafcet-Engine
            engine.GrafcetCyclic()
            #DO- write all outputs from the Grafcet-Engine to the hardware
            f.seek(do_ofs) # offset of the DO is 81
            do_array=bytearray(do_len)
            for i in range(do_len):
                do_array[i]=engine.read_uint8(grafcet_do_ofs+i)
            f.write(bytes(do_array))
            # each second send the monitoring info
            if time.time()-timeOldStatusMessage>1: 
                print("Grafcet-Run={}, Cyclic time={}ms, di_array={}, do_array={}".format(engine.GrafcetEngineStatus.GrafcetRun, engine.GrafcetEngineStatus.PlcCyclicTimeMs, di_array,do_array))
                timeOldStatusMessage=time.time()
except KeyboardInterrupt: #Ctrl+C pressed? Then exit
    pass
engine.exit() #exit and close Grafcet-Engine
print("End of Python script")


